#include "matrix.h"
#include "point.h"
#include "quater.h"

Matrix::Matrix()
{
  SetIdentity();
}

Matrix::~Matrix()
{
}

VOID Matrix::SetNull()
{
  memset(m_m, 0, sizeof(MATRIX4D));
}

VOID Matrix::SetIdentity()
{
  UINT_32 i;

  SetNull();

  for ( i = 0; i < 16; i += 5 )
    {
      m_m[i] = 1.0;
    }
}

VOID Matrix::SetCross(const Vector &v)
{
  SetNull();

  m_m[4] = -(v(2));
  m_m[1] = (v(2));
  m_m[8] = (v(1));
  m_m[2] = -(v(1));
  m_m[9] = -(v(0));
  m_m[6] = (v(0));
}

VOID Matrix::SetTranslate(const Vector &v)
{
  SetIdentity();

  m_m[12] = v(0);
  m_m[13] = v(1);
  m_m[14] = v(2);
}
/*
VOID Matrix::SetQuaternion(const Quater &q)
{
  FLOAT x, y, z, w;

  x = (&q)->GetSafe(QI);
  y = (&q)->GetSafe(QJ);
  z = (&q)->GetSafe(QK);
  w = (&q)->GetSafe(QS);
  
  m_pv[0][0] = (w*w)+(x*x)-(y*y)-(z*z);
  m_pv[0][1] = 2*x*y-2*w*z;
  m_pv[0][2] = 2*x*z+2*w*y;
  m_pv[0][3] = 0;
  
  m_pv[1][0] = 2*x*y+2*w*z;
  m_pv[1][1] = (w*w)-(x*x)+(y*y)-(z*z);
  m_pv[1][2] = 2*y*z-2*w*x;
  m_pv[1][3] = 0;
  
  m_pv[2][0]= 2*x*z-2*w*y;
  m_pv[2][1] = 2*y*z+2*w*x;
  m_pv[2][2] = (w*w)-(x*x)-(y*y)+(z*z);
  m_pv[2][3] = 0;
  
  m_pv[3][0] = 0;
  m_pv[3][1] = 0;
  m_pv[3][2] = 0;
  m_pv[3][3] = (w*w)+(x*x)+(y*y)+(z*z);
}

VOID Matrix::SetTwist(Twist *pTwist, DOUBLE dTheta)
{
  UINT_32 i;
  Matrix I;
  Vector v, vW, vV;

  // Twists for Kinematics

  vV = pTwist->GetV();
  vW = pTwist->GetW();

  if ( vW.Magnitude() == 0.0f )
    {
      SetIdentity();
      v = vV * dTheta;
    }
  else
    {
      I.SetIdentity();
      SetRotate(vW, dTheta);
      v = ((I - (*this)) * (vW * vV)) + (vW * (vW.Dot(vV) * dTheta));
    }

  for ( i = 0; i < R3; i++)
    {
      m_pv[i][3] = (&v)->GetSafe(i);
    }
}


VOID Matrix::SetTwistInverse(Matrix &m)
{
  UINT_32 i;
  Matrix mT, mR;

  // Assume m = T*R a translation times a rotation (a screw)
  // Then m-1 = (R-1)*(T-1) = (Rt) * (T-1)

  mR = m;
  mT.SetIdentity();
  for ( i = 0; i < 3; i++ )
    {
      mT.m_pv[i][3] = -m.m_pv[i][3];
      mR.m_pv[i][3] = 0.0f;
    }
  mR.Transpose();

  (*this) = mR * mT;
}
*/
VOID Matrix::SetRotate(const Vector &v, DOUBLE dTheta)
{
  // Rodrigues Formula

  Vector u;
  Matrix I, m;
  FLOAT sint, cost;

  sint = (FLOAT)sin(dTheta);
  cost = (FLOAT)cos(dTheta);

  u = v;
  u.Normalize();
  m.SetCross(u);
  I.SetIdentity();

  (*this) = I + (m*sint) + (m*m*(1.0f - cost));
}  

VOID Matrix::SetRotate(const Vector &vEx, const Vector &vEy, const Vector &vEz)
{
  SetIdentity();

  // These are columns
  m_m[0] = vEx(0);
  m_m[1] = vEx(1);
  m_m[2] = vEx(2);
  m_m[4] = vEy(0);
  m_m[5] = vEy(1);
  m_m[6] = vEy(2);
  m_m[8] = vEz(0);
  m_m[9] = vEz(1);
  m_m[10] = vEz(2);
}

VOID Matrix::SetRotate(const Quater &q)
{
   Quater nq = q;
   nq.Normalize();

   FLOAT
      x2 = nq(0)*nq(0),
      y2 = nq(1)*nq(1),
      z2 = nq(2)*nq(2),
      w2 = nq(3)*nq(3),
      xy = nq(0)*nq(1),
      xz = nq(0)*nq(2),
      xw = nq(0)*nq(3),
      yz = nq(1)*nq(2),
      yw = nq(1)*nq(3),
      zw = nq(2)*nq(3);

   SetIdentity();

   // Diagonal Elements
   m_m[0]  = w2 + x2 - y2 - z2;
   m_m[5]  = w2 - x2 + y2 - z2;
   m_m[10] = w2 - x2 - y2 + z2;
   m_m[15] = w2 + x2 + y2 + z2;  // Should be = 1

   // Off-Diagonal Elements
   m_m[1]  = 2*xy + 2*zw;
   m_m[4]  = 2*xy - 2*zw;
   m_m[2]  = 2*xz - 2*yw;
   m_m[8]  = 2*xz + 2*yw;
   m_m[6]  = 2*yz + 2*xw;
   m_m[9]  = 2*yz - 2*xw;
}

/*
VOID Matrix::SetRotate(Vector v, DOUBLE dAlpha)
{
  // Euler Angles

  Matrix m;
  DOUBLE dTheta, dPhi;
  FLOAT f;

  if ( v[X] == 0 )
    {
      if ( v[Y] < 0 )
	{
	  dTheta = -PI/2;
	}
      else if( v[Y] > 0 )
	{
	  dTheta = PI/2;
	}
      else
	{
	  dTheta = 0.0;
	}
    }
  else
    {
      dTheta = atan(v[Y]/v[X]);
    }

  dPhi = ((f=sqrt(v[X]*v[X] + v[Y]*v[Y]))==0) ? (PI/2) : (atan(v[Z]/f));

  m.SetRotateZ(-dTheta);
  (*this) = m*(*this);
  m.SetRotateY(-(PI/2 - dPhi));
  (*this) = m*(*this);
  m.SetRotateZ(dAlpha);
  (*this) = m*(*this);
  m.SetRotateY(PI/2 - dPhi);
  (*this) = m*(*this);
  m.SetRotateZ(dTheta);
  (*this) = m*(*this);
}  
*/

VOID Matrix::SetRotateX(DOUBLE dTheta)
{
  SetIdentity();
  m_m[5] = (FLOAT)cos(dTheta);
  m_m[9] = (FLOAT)(-sin(dTheta));
  m_m[6] = (FLOAT)sin(dTheta);
  m_m[10] = (FLOAT)cos(dTheta);
}

VOID Matrix::SetRotateY(DOUBLE dTheta)
{
  SetIdentity();
  m_m[0] = (FLOAT)cos(dTheta);
  m_m[8] = (FLOAT)sin(dTheta);
  m_m[2] = (FLOAT)(-sin(dTheta));
  m_m[10] = (FLOAT)cos(dTheta);
}

VOID Matrix::SetRotateZ(DOUBLE dTheta)
{
  SetIdentity();
  m_m[0] = (FLOAT)cos(dTheta);
  m_m[4] = (FLOAT)(-sin(dTheta));
  m_m[1] = (FLOAT)sin(dTheta);
  m_m[5] = (FLOAT)cos(dTheta);
}

VOID Matrix::SetRotateZ(FLOAT sint, FLOAT cost)
{
  SetIdentity();
  m_m[0] = cost;
  m_m[4] = -sint;
  m_m[1] = sint;
  m_m[5] = cost;
}

VOID Matrix::SetScale(const Vector &v)
{
  SetIdentity();
  m_m[0] = v(0);
  m_m[5] = v(1);
  m_m[10] = v(2);
}

VOID Matrix::SetScale(const FLOAT f)
{
  SetIdentity();
  m_m[0] = f;
  m_m[5] = f;
  m_m[10] = f;
}

VOID Matrix::SetShearXY(const Vector &v)
{
  ASSERT( v(2) != 0.0 );

  SetIdentity();
  m_m[8] = v(0)/v(2);
  m_m[9] = v(1)/v(2);
}

VOID Matrix::SetShearYZ(const Vector &v)
{
  ASSERT( v(0) != 0.0 );

  SetIdentity();
  m_m[1] = v(1)/v(0);
  m_m[2] = v(2)/v(0);
}

VOID Matrix::SetShearXZ(const Vector &v)
{
  ASSERT( v(1) != 0.0 );

  SetIdentity();
  m_m[4] = v(0)/v(1);
  m_m[6] = v(2)/v(1);
}

//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 6: Implement Perspective Projection and its inverse
// 
VOID Matrix::SetPerspectiveWarp(const FLOAT fZfront)
{
  ASSERT( 0.0 > fZfront && fZfront > -1.0f );
  
  SetIdentity();
}

VOID Matrix::SetPerspectiveWarpInverse(const FLOAT fZfront)
{
  ASSERT( 0.0 > fZfront && fZfront > -1.0f );
  
  SetIdentity();
}
// 
// END  : ASSIGNMENT 6: Implement Perspective Projection and its inverse
//////////////////////////////////////////////////////////////////////////////

VOID Matrix::SetViewPort(const Vector &vVPMin, const Vector &vVPMax, 
			 const Vector &vWWMin, const Vector &vWWMax, 
			 FLOAT fDYDX/* = 1.0*/)
{
  Matrix mTransWW, mTransVP, mScaleNonuniform, mScale;
  Vector vVP, vWW, vScale, v;
  FLOAT fScale;

  vWW = (vWWMin + vWWMax) / 2.0;
  vWW[2] = 0.0;
  mTransWW.SetTranslate(-vWW);

  vScale.Set(1.0, fDYDX, 1.0);
  mScaleNonuniform.SetScale(vScale);

  vWW = vWWMax - vWWMin;
  vWW[1] = fDYDX*vWW[1];
  vWW[2] = 0.0;
  vVP = vVPMax - vVPMin;
  vVP[2] = 0.0;
  
  if ( vWW[0] != 0.0 && vWW[1] != 0.0 )
    {
      fScale = MIN(vVP[0]/vWW[0], vVP[1]/vWW[1]);
    }
  else if ( vWW[0] != 0.0 )
    {
      fScale = vVP[0]/vWW[0];
    }
  else if ( vWW[1] != 0.0 )
    {
      fScale = vVP[1]/vWW[1];
    }
  else
    {
      fScale = 1.0;
    }

  vScale.Set(fScale, fScale, fScale);
  mScale.SetScale(vScale);

  v = (vVPMin + vVPMax) / 2.0;
  v[2] = -1.0;
  mTransVP.SetTranslate(v);

  *this = mTransVP * mScale * mScaleNonuniform * mTransWW;
}

FLOAT Matrix::Trace() const
{
  return ( m_m[0] + m_m[5] + m_m[10] );
}

FLOAT Matrix::Determinant() const
{
  return  ( ( -m_m[ 3] * ( ( (m_m[ 4] * m_m[ 9] * m_m[14]) +
			     (m_m[ 8] * m_m[13] * m_m[ 6]) +
			     (m_m[12] * m_m[ 5] * m_m[10]) ) -
			   ( (m_m[ 6] * m_m[ 9] * m_m[12]) +
			     (m_m[10] * m_m[13] * m_m[ 4]) +
			     (m_m[14] * m_m[ 5] * m_m[ 8]) ) ) ) +
	    (  m_m[ 7] * ( ( (m_m[ 0] * m_m[ 9] * m_m[14]) +
			     (m_m[ 8] * m_m[13] * m_m[ 2]) +
			     (m_m[12] * m_m[ 1] * m_m[10]) ) -
			   ( (m_m[ 2] * m_m[ 9] * m_m[12]) +
			     (m_m[10] * m_m[13] * m_m[ 0]) +
			     (m_m[14] * m_m[ 1] * m_m[ 8]) ) ) ) +
	    ( -m_m[11] * ( ( (m_m[ 0] * m_m[ 5] * m_m[14]) +
			     (m_m[ 4] * m_m[13] * m_m[ 2]) +
			     (m_m[12] * m_m[ 1] * m_m[ 6]) ) -
			   ( (m_m[ 2] * m_m[ 5] * m_m[12]) +
			     (m_m[ 6] * m_m[13] * m_m[ 0]) +
			     (m_m[14] * m_m[ 1] * m_m[ 4]) ) ) ) +
	    (  m_m[15] * ( ( (m_m[ 0] * m_m[ 5] * m_m[10]) +
			     (m_m[ 4] * m_m[ 9] * m_m[ 2]) +
			     (m_m[ 8] * m_m[ 1] * m_m[ 6]) ) -
			   ( (m_m[ 2] * m_m[ 5] * m_m[ 8]) +
			     (m_m[ 6] * m_m[ 9] * m_m[ 0]) +
			     (m_m[10] * m_m[ 1] * m_m[ 4]) ) ) ) );
}

VOID Matrix::Transpose()
{
  UINT_32 i, j;
  FLOAT f;

  for ( i = 0; i < (4-1); i++ )
    {
      for ( j = i+1; j < 4; j++)
	{
	  f = m_m[Index(i,j)];
	  m_m[Index(i,j)] = m_m[Index(j,i)];
	  m_m[Index(j,i)] = f;
	}
    }
}

Matrix &Matrix::operator=(const Matrix& m)
{
  memcpy(m_m, m.m_m, sizeof(MATRIX4D));

  return (*this);
}

Matrix &Matrix::operator+=(const Matrix& m)
{
  UINT_32 i;

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] + m.m_m[i];
    }

  return (*this);
}

Matrix &Matrix::operator-=(const Matrix& m)
{
  UINT_32 i;

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] - m.m_m[i];
    }

  return (*this);
}

Matrix &Matrix::operator*=(const FLOAT f)
{
  UINT_32 i;

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] * f;
    }

  return (*this);
}

Matrix &Matrix::operator/=(const FLOAT f)
{
  UINT_32 i;

  ASSERT( f != 0.0 );

  for ( i = 0; i < 16; i++ )
    {
      m_m[i] = m_m[i] / f;
    }

  return (*this);
}

Matrix Matrix::operator*(const Matrix& m) const
{
  UINT_32 i, j, k;
  UINT_32 i4, k4;

  Matrix mResult;

  i4 = 0;
  for ( i = 0; i < 4; i++ ) // Col Selector
    {
      for ( j = 0; j < 4; j++ ) // Row Selector
	{
	  mResult.m_m[i4+j] = 0.0;
	  k4 = 0;
	  for ( k = 0; k < 4; k++ ) // Iterator
	    {
	      mResult.m_m[i4+j] += m_m[k4+j] * m.m_m[i4+k];
	      k4 += 4;
	    }
	}
      i4 += 4;
    }

  return mResult;
}

Vector Matrix::operator*(const Vector &v) const
{
  UINT_32 i, j, j4;
  Vector vResult;

  for ( i = 0; i < 4; i++ )
    {
      vResult[i] = 0.0f;
      j4 = 0;
      for ( j = 0; j < 4; j++ )
	{
	  vResult[i] += m_m[i + j4] * v(j);
	  j4 += 4;
	}
    }

  return vResult;
}

Vector operator*(const Vector &v, const Matrix &m)
{
  UINT_32 i, j, k;
  Vector vResult;
  MATRIX4D mTemp;

  m.Get(mTemp);

  k = 0;
  for ( i = 0; i < 4; i++ )
    {
      vResult[i] = 0.0f;
      for ( j = 0; j < 4; j++ )
	{
	  vResult[i] += mTemp[k] * v(j);
	  k++;
	}
    }

  return vResult;
}

Point Matrix::operator*(const Point &pt) const
{
  UINT_32 i, j, j4;
  Point ptResult;

  for ( i = 0; i < 4; i++ )
    {
      ptResult[i] = 0.0f;
      j4 = 0;
      for ( j = 0; j < 4; j++ )
	{
	  ptResult[i] += m_m[i + j4] * pt(j);
	  j4 += 4;
	}
    }

  return ptResult;
}

VOID Matrix::Display() const
{
  UINT_32 i, j;

  for ( i = 0; i < 4; i++ )
    {
      printf("| ");
      for ( j = 0; j < 4; j++ )
	{
	  printf("%1.3f ", m_m[Index(i,j)]);
	}
      printf("|\n");
    }
}




