%{
#include "slideparser.h"
#include "slide_.tab.h"

#define yylval slide_lval
extern YYSTYPE yylval;    // holds start line and column of token.
#define yylloc slide_lloc
extern YYLTYPE yylloc;    // holds start line and column of token.
CScanner *slide_gpCScanner = NULL;

#undef yywrap
#define yywrap() slide_gpCScanner->PopFile()

#define FLEX_DEBUG 1

	// Define macro to return token and update column number.
#define ReturnToken(x) \
	slide_gpCScanner->ConsumeToken(); \
	return (x)

#define ReturnTokenInt() \
	slide_gpCScanner->ScanInteger(yytext); \
	return tokCONST_INT

#define ReturnTokenFloat() \
	slide_gpCScanner->ScanFloat(yytext); \
	return tokCONST_FLOAT

#define ReturnTokenString() \
	slide_gpCScanner->ScanString(); \
	return tokCONST_STRING

#define ReturnTokenTclBlock() \
	slide_gpCScanner->ScanTclBlock(); \
	return tokTCL_BLOCK

#define ReturnTokenIdentifier() \
	slide_gpCScanner->ScanIdentifier(yytext); \
	return tokIDENTIFIER

#define TABSIZE 8	// Size of tab.

%}

OCT     [0-7]
DIGIT	[0-9]
HEX     [0-9A-Fa-f]
LETTER	[A-Za-z_]
IDTAIL	[A-Za-z0-9_]
ID	{LETTER}{IDTAIL}*
STR     [^\0]|"\\\""
SPACE	[ \t\f\v\a]

FLOAT	"-"?({DIGIT}*\.)?{DIGIT}+([eE][-+]?{DIGIT}+)?

%x COMMENT_STATE
%x STRING_STATE
%x TCL_BLOCK_STATE
%x INCLUDE_STATE

%%

<COMMENT_STATE>"(*"	{slide_gpCScanner->CommentStart();}
<COMMENT_STATE>"*)"	{
			  if ( slide_gpCScanner->CommentEnd() )
			    {
			      BEGIN 0;
			    }
			}
<COMMENT_STATE>"\n"	{slide_gpCScanner->ConsumeNewLine();}
<COMMENT_STATE>[*(]	{slide_gpCScanner->ConsumeToken();}
<COMMENT_STATE>[^*(\n]+	{slide_gpCScanner->ConsumeToken();}

<STRING_STATE>"\""	{slide_gpCScanner->StringEnd(); BEGIN 0; ReturnTokenString();}
<STRING_STATE>"\n"	{slide_gpCScanner->StringAppendNewLine();}
<STRING_STATE>"\\\n"	{slide_gpCScanner->ConsumeNewLine();}
<STRING_STATE>[^"\\\n]+	{slide_gpCScanner->StringAppend(yytext);}

<TCL_BLOCK_STATE>"{"	{slide_gpCScanner->TclBlockStart();}
<TCL_BLOCK_STATE>"}"	{
			  if ( slide_gpCScanner->TclBlockEnd() )
			    {
			      BEGIN 0;
			      ReturnTokenTclBlock();
			    }
			}
<TCL_BLOCK_STATE>"\n"	{slide_gpCScanner->TclBlockAppendNewLine();}
<TCL_BLOCK_STATE>"\\\n"	{slide_gpCScanner->ConsumeNewLine();}
<TCL_BLOCK_STATE>"\\{"	{slide_gpCScanner->TclBlockAppend(yytext);}
<TCL_BLOCK_STATE>"\\}"	{slide_gpCScanner->TclBlockAppend(yytext);}
<TCL_BLOCK_STATE>[^{}\\\n]+	{slide_gpCScanner->TclBlockAppend(yytext);}

<INCLUDE_STATE>"\""	{
			  slide_gpCScanner->FileNameEnd(); 
			  BEGIN 0; 
			  slide_gpCScanner->PushFile(slide_gpCScanner->ScanFileName());
			}
<INCLUDE_STATE>"\n"	        {slide_gpCScanner->FileNameAppendNewLine();}
<INCLUDE_STATE>"\\\n"	        {slide_gpCScanner->ConsumeNewLine();}
<INCLUDE_STATE>[^"\\\n]+	{slide_gpCScanner->FileNameAppend(yytext);}

\n		{slide_gpCScanner->ConsumeNewLine();}
{SPACE}+	{slide_gpCScanner->ConsumeToken();}

"tclinit"	{ReturnToken(rwTCL_INIT);}
"tclupdate"	{ReturnToken(rwTCL_UPDATE);}
"tcltag"	{ReturnToken(rwTCL_TAG);}
"window"       	{ReturnToken(rwWINDOW);}
"endwindow"	{ReturnToken(rwEND_WINDOW);}
"viewport"	{ReturnToken(rwVIEWPORT);}
"endviewport"	{ReturnToken(rwEND_VIEWPORT);}
"origin"	{ReturnToken(rwORIGIN);}
"size"		{ReturnToken(rwSIZE);}
"background"	{ReturnToken(rwBACKGROUND);}
"camera"	{ReturnToken(rwCAMERA);}
"endcamera"	{ReturnToken(rwEND_CAMERA);}
"projection"	{ReturnToken(rwPROJECTION);}
"frustum"	{ReturnToken(rwFRUSTUM);}
"light"       	{ReturnToken(rwLIGHT);}
"endlight"	{ReturnToken(rwEND_LIGHT);}
"type"       	{ReturnToken(rwTYPE);}
"deaddistance"	{ReturnToken(rwDEAD_DISTANCE);}
"falloff"	{ReturnToken(rwFALL_OFF);}
"angularfalloff"       	{ReturnToken(rwANGULAR_FALL_OFF);}
"render"       	{ReturnToken(rwRENDER);}
"endrender"	{ReturnToken(rwEND_RENDER);}
"group"		{ReturnToken(rwGROUP);}
"endgroup"	{ReturnToken(rwEND_GROUP);}
"instance"	{ReturnToken(rwINSTANCE);}
"endinstance"	{ReturnToken(rwEND_INSTANCE);}
"camerainstance"	{ReturnToken(rwINSTANCE);}
"endcamerainstance"	{ReturnToken(rwEND_INSTANCE);}
"lightinstance"		{ReturnToken(rwINSTANCE);}
"endlightinstance"	{ReturnToken(rwEND_INSTANCE);}
"translate"	{ReturnToken(rwTRANSLATE);}
"scale"		{ReturnToken(rwSCALE);}
"rotate"	{ReturnToken(rwROTATE);}
"lookat"	{ReturnToken(rwLOOK_AT);}
"endlookat"	{ReturnToken(rwEND_LOOK_AT);}
"eye"		{ReturnToken(rwEYE);}
"target"	{ReturnToken(rwTARGET);}
"up"		{ReturnToken(rwUP);}
"object"	{ReturnToken(rwOBJECT);}
"endobject"	{ReturnToken(rwEND_OBJECT);}
"face"		{ReturnToken(rwFACE);}
"endface"	{ReturnToken(rwEND_FACE);}
"point"		{ReturnToken(rwPOINT);}
"endpoint"	{ReturnToken(rwEND_POINT);}
"surface"	{ReturnToken(rwSURFACE);}
"endsurface"	{ReturnToken(rwEND_SURFACE);}
"color"		{ReturnToken(rwCOLOR);}
"reflectivity"	{ReturnToken(rwREFLECTIVITY);}
"exponent"	{ReturnToken(rwEXPONENT);}
"metallic"	{ReturnToken(rwMETALLIC);}
"SLF_INHERIT"	{ReturnToken(rwSLF_INHERIT);}
"id"		{ReturnToken(rwID);}
"SLF_OFF"	{ReturnToken(rwSLF_OFF);}
"SLF_ON"	{ReturnToken(rwSLF_ON);}
"solid"		{ReturnToken(rwSOLID);}
"SLF_SOLID"	{ReturnToken(rwSLF_SOLID);}
"SLF_HOLLOW"	{ReturnToken(rwSLF_HOLLOW);}
"shading"	{ReturnToken(rwSHADING);}
"SLF_WIRE"	{ReturnToken(rwSLF_WIRE);}
"SLF_FLAT"	{ReturnToken(rwSLF_FLAT);}
"SLF_GOURAUD"	{ReturnToken(rwSLF_GOURAUD);}
"SLF_PHONG"	{ReturnToken(rwSLF_PHONG);}
"lod"		{ReturnToken(rwLOD);}
"SLF_BOUND"	{ReturnToken(rwSLF_BOUND);}
"SLF_EDGES"	{ReturnToken(rwSLF_EDGES);}
"SLF_FULL"	{ReturnToken(rwSLF_FULL);}
"SLF_PARALLEL"	{ReturnToken(rwSLF_PARALLEL);}
"SLF_PERSPECTIVE"	{ReturnToken(rwSLF_PERSPECTIVE);}
"normal"	{ReturnToken(rwNORMAL);}
"texture"	{ReturnToken(rwTEXTURE);}
"("		{ReturnToken('(');}
")"		{ReturnToken(')');}
"."		{ReturnToken('.');}

"(*"		{slide_gpCScanner->CommentStart(); BEGIN COMMENT_STATE;}
"*)"		{
		  slide_gpCScanner->HandleError(0, 0, "Mismatched comment parentheses");
		  slide_gpCScanner->ConsumeToken();
		}
"#".*		{slide_gpCScanner->ConsumeToken();}

"\""		{slide_gpCScanner->StringStart(); BEGIN STRING_STATE;}

"{"		{slide_gpCScanner->TclBlockStart(); BEGIN TCL_BLOCK_STATE;}
"}"		{
		  slide_gpCScanner->HandleError(0, 0, "Mismatched brace");
		  slide_gpCScanner->ConsumeToken();
		}

"include"{SPACE}+"\"" {slide_gpCScanner->FileNameStart(); BEGIN INCLUDE_STATE;}

{FLOAT}		{ReturnTokenFloat();}

{ID}		{ReturnTokenIdentifier();}

.		{
		  slide_gpCScanner->HandleError(0, 0, "Unknown character \"%s\"", 
		                               yylloc.text);
		  slide_gpCScanner->ConsumeToken();
		}

%%

//"general"	{ReturnToken(rwGENERAL);}

VOID slide_BeginInitial()
{
  BEGIN 0;
}

YY_BUFFER_STATE slide_GetCurrentBuffer()
{
  return YY_CURRENT_BUFFER;
}

YY_BUFFER_STATE slide_CreateBuffer(FILE *fpFile)
{
  return yy_create_buffer(fpFile, YY_BUF_SIZE);
}

VOID slide_DeleteBuffer(YY_BUFFER_STATE yyBufferState)
{
  yy_delete_buffer(yyBufferState);
}

VOID slide_SwitchToBuffer(YY_BUFFER_STATE yyBufferState)
{
  yy_switch_to_buffer(yyBufferState);
}

VOID slide_SetValueInt(int i)
{
  yylval.i = i;
}

VOID slide_SetValueFloat(float f)
{
  yylval.f = f;
}

VOID slide_SetValueString(CHAR *pc)
{
  yylval.str = pc;
}








