#ifndef __SLIDECAMERA_H__
#define __SLIDECAMERA_H__

#include "slidenode.h"
#include "slidepath.h"

//////////////////////////////////////////////////////////////////////
// CSLIDECamera Class
//

class CSLIDECamera : public CSLIDENode
{
//////////////////////////////////////////////////
// Enumerated Types
//

//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDECamera();
  virtual ~CSLIDECamera();
  virtual VOID Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
		    CHAR *pcID);
  virtual VOID Uninit();

  virtual BOOL Initialized();

  virtual BOOL CheckSemantics(CSLIDEWorld *pCSLIDEWorld);

  virtual BOOL Valid();

//////////////////////////////////////////////////
// Geometric Computations
//
public:
  virtual CSLIDECamera *FindCamera(CListIterInt *piterIDs, 
				   CListInt *plistCSLIDEInstance);
  virtual CSLIDELight *FindLight(CListIterInt *piterIDs, 
				 CListInt *plistCSLIDEInstance);

protected:
  VOID CheckDynamic();
  VOID UpdateState();
//////////////////////////////////////////////////////////////////////////////
// START: ASSIGNMENT 6: Implement Perspective Projection and its inverse
// 
  VOID CalculateProjectionMatrix();
// 
// END  : ASSIGNMENT 6: Implement Perspective Projection and its inverse
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////
// Rendering
//
public:
  virtual VOID Render(const Matrix &mGvp_proj, 
		      const Matrix &mGproj_vrc, const Matrix &mGvrc_proj, 
		      const Matrix &mGvrc_obj, const Matrix &mGobj_vrc, 
		      CBound *pCBoundClip, 
		      CListInt *plistLights,
		      CSLIDESurface *pCSLIDESurface,
		      SLIDEShadingType eShading, 
		      SLIDELODType eLOD);

//////////////////////////////////////////////////
// Output Functions
//
public:
  virtual VOID WriteSLIDE(FILE *fpOutputFile);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline const Vector &GetFrustumMin();
  inline VOID SetFrustumMin(const Vector &v);
  inline VOID SetFrustumMin(DynVector *pdynv);
  inline const Vector &GetFrustumMax();
  inline VOID SetFrustumMax(const Vector &v);
  inline VOID SetFrustumMax(DynVector *pdynv);
  inline VOID SetProjection(SLIDEProjectionType eProjection);
  inline VOID SetProjection(DynInt *pdyniProjection);

  virtual inline NodeType GetNodeType();

  // This is the Projection matrix
  inline const Matrix &GetVRCToProjection();
  inline const Matrix &GetProjectionToVRC();
  // This is for the crystal ball interface to see what we are focusing on
  inline const Matrix GetVRPToVRC();
  inline const Matrix GetVRCToVRP();

//////////////////////////////////////////////////
// Class Variables
//
public:

//////////////////////////////////////////////////
// Member Variables
//
protected:
  DynInt m_dyniProjection;
  DynVector m_vFrustumMin;
  DynVector m_vFrustumMax;

  Matrix m_mGproj_vrc;
  Matrix m_mGvrc_proj;
};

//////////////////////////////////////////////////////////////////////////////
// Inlined Functions
//
inline const Vector &CSLIDECamera::GetFrustumMin()
{
  return m_vFrustumMin;
}

inline VOID CSLIDECamera::SetFrustumMin(const Vector &v)
{
  m_vFrustumMin = v;
}

inline VOID CSLIDECamera::SetFrustumMin(DynVector *pdynv)
{
  m_vFrustumMin.SetAll(pdynv);
}

inline const Vector &CSLIDECamera::GetFrustumMax()
{
  return m_vFrustumMax;
}

inline VOID CSLIDECamera::SetFrustumMax(const Vector &v)
{
  m_vFrustumMax = v;
}

inline VOID CSLIDECamera::SetFrustumMax(DynVector *pdynv)
{
  m_vFrustumMax.SetAll(pdynv);
}

inline VOID CSLIDECamera::SetProjection(SLIDEProjectionType eProjection)
{
  ASSERT( eProjection < slidepjtSize );

  m_dyniProjection.SetValue(eProjection);
}

inline VOID CSLIDECamera::SetProjection(DynInt *pdyniProjection)
{
  ASSERT( pdyniProjection != NULL );
  ASSERT( 0 <= pdyniProjection->GetValue() &&
	  pdyniProjection->GetValue() < slidepjtSize );

  m_dyniProjection.Set(pdyniProjection);
}

inline CSLIDENode::NodeType CSLIDECamera::GetNodeType()
{
  return ntCamera;
}

inline const Matrix &CSLIDECamera::GetVRCToProjection()
{
  return m_mGproj_vrc;
}

inline const Matrix &CSLIDECamera::GetProjectionToVRC()
{
  return m_mGvrc_proj;
}

inline const Matrix CSLIDECamera::GetVRPToVRC()
{
  Vector vTrans;
  Matrix mTrans;

  vTrans = (m_vFrustumMax + m_vFrustumMin) / 2.0;
  vTrans[Z] = -1.0;
  mTrans.SetTranslate(vTrans);

  return mTrans;
}

inline const Matrix CSLIDECamera::GetVRCToVRP()
{
  Vector vTrans;
  Matrix mTrans;

  vTrans = (m_vFrustumMax + m_vFrustumMin) / (-2.0);
  vTrans[Z] = 1.0;
  mTrans.SetTranslate(vTrans);

  return mTrans;
}

//////////////////////////////////////////////////////////////////////
// CSLIDECameraPath Class
//

class CSLIDECameraPath : public CSLIDEPath
{
//////////////////////////////////////////////////
// Initialization
//
public:
  CSLIDECameraPath();
  virtual ~CSLIDECameraPath();

  virtual CSLIDENode *FindNode(CSLIDENode *pRoot, CListIterInt *piterIDs, 
			       CListInt *plistCSLIDEInstance);

//////////////////////////////////////////////////
// Accessor Functions
//
public:
  inline CSLIDECamera *GetCamera();

//////////////////////////////////////////////////
// Member Variables
//
protected:
};

//////////////////////////////////////////////////////////////////////
// Inlined Functions
//

inline CSLIDECamera *CSLIDECameraPath::GetCamera()
{
  return (CSLIDECamera*)m_pNode;
}

#endif // __SLIDECAMERA_H__


