#ifndef __STDINPUT_H__
#define __STDINPUT_H__

// This is an attempt to abstract away all types of standard input devices
// such as keyboards and mice with buttons

#include "genlib.h"

#include <X11/keysym.h>

//////////////////////////////////////////////////////////////////////////////
// The Mouse
//

// Buttons
#define GLSI_INVALID_BUTTON		0
#define GLSI_LEFT_BUTTON		1
#define GLSI_MIDDLE_BUTTON		2
#define GLSI_RIGHT_BUTTON		3

// Button States
#define GLSI_DOWN			0
#define GLSI_UP				1

// Iconification States
#define GLSI_ICON			0
#define GLSI_ACTIVE    			1


//////////////////////////////////////////////////////////////////////////////
// The Keyboard
//

#define GLSI_KEY_VOIDSYMBOL         0xFFFFFF  // void symbol

#define GLSI_KEY_BACKSPACE          0xFF08    // back space, back char
#define GLSI_KEY_TAB                0xFF09
#define GLSI_KEY_LINEFEED           0xFF0A    // Linefeed, LF
#define GLSI_KEY_CLEAR              0xFF0B
#define GLSI_KEY_RETURN             0xFF0D    // Return, enter
#define GLSI_KEY_PAUSE              0xFF13    // Pause, hold
#define GLSI_KEY_SCROLL_LOCK        0xFF14
#define GLSI_KEY_ESCAPE             0xFF15
#define GLSI_KEY_DELETE             0xFFFF    // Delete, rubout

// International & multi-key character composition

#define GLSI_KEY_MULTI_KEY          0xFF20    // Multi-key char compose 

// Cursor control & motion

#define GLSI_KEY_HOME               0xFF50	       
#define GLSI_KEY_LEFT               0xFF51    // Move left, left arrow
#define GLSI_KEY_UP                 0xFF52    // Move up, up arrow
#define GLSI_KEY_RIGHT              0xFF53    // Move right, right arrow
#define GLSI_KEY_DOWN               0xFF54    // Move down, down arrow
#define GLSI_KEY_PRIOR              0xFF55    // Prior, previous
#define GLSI_KEY_NEXT               0xFF56    // Next
#define GLSI_KEY_END                0xFF57    // EOL
#define GLSI_KEY_BEGIN              0xFF58    // BOL


// Misc Functions
#define GLSI_KEY_SELECT             0xFF60    // Select, mark 
#define GLSI_KEY_PRINT              0xFF61
#define GLSI_KEY_EXECUTE            0xFF62    // Execute, run, do 
#define GLSI_KEY_INSERT             0xFF63    // Insert, insert here 
#define GLSI_KEY_UNDO               0xFF65    // Undo, oops 
#define GLSI_KEY_REDO               0xFF66    // redo, again 
#define GLSI_KEY_MENU               0xFF67		
#define GLSI_KEY_FIND               0xFF68    // Find, search 
#define GLSI_KEY_CANCEL             0xFF69    // Cancel, stop, abort, exit 
#define GLSI_KEY_HELP               0xFF6A    // Help, ? 
#define GLSI_KEY_BREAK              0xFF6B	
#define GLSI_KEY_MODE_SWITCH        0xFF7E    // Character set switch 
#define GLSI_KEY_SCRIPT_SWITCH      0xFF7E    // Alias for mode_switch 
#define GLSI_KEY_NUM_LOCK           0xFF7F		

// Keypad Functions, keypad numbers cleverly chosen to map to ascii 
#define GLSI_KEY_SPACE              0xFF80    // space 
//#define GLSI_KEY_TAB                0xFF89		
#define GLSI_KEY_ENTER              0xFF8D    // enter 
//#define GLSI_KEY_F1                 0xFF91    // PF1, KP_A, ... 
//#define GLSI_KEY_F2                 0xFF92		
//#define GLSI_KEY_F3                 0xFF93		
//#define GLSI_KEY_F4                 0xFF94		
#define GLSI_KEY_EQUAL              0xFFBD    // equals 
#define GLSI_KEY_MULTIPLY           0xFFAA		
#define GLSI_KEY_ADD                0xFFAB		
#define GLSI_KEY_SEPARATOR          0xFFAC    // separator, often comma 
#define GLSI_KEY_SUBTRACT           0xFFAD		
#define GLSI_KEY_DECIMAL            0xFFAE		
#define GLSI_KEY_DIVIDE             0xFFAF		

#define GLSI_KEY_0                  0xFFB0			
#define GLSI_KEY_1                  0xFFB1			
#define GLSI_KEY_2                  0xFFB2			
#define GLSI_KEY_3                  0xFFB3			
#define GLSI_KEY_4                  0xFFB4			
#define GLSI_KEY_5                  0xFFB5			
#define GLSI_KEY_6                  0xFFB6			
#define GLSI_KEY_7                  0xFFB7			
#define GLSI_KEY_8                  0xFFB8			
#define GLSI_KEY_9                  0xFFB9			

/*
 * Auxilliary Functions; note the duplicate definitions for left and right
 * function keys;  Sun keyboards and a few other manufactures have such
 * function key groups on the left and/or right sides of the keyboard.
 * We've not found a keyboard with more than 35 function keys total.
 */

#define GLSI_KEY_F1                 0xFFBE			
#define GLSI_KEY_F2                 0xFFBF			
#define GLSI_KEY_F3                 0xFFC0			
#define GLSI_KEY_F4                 0xFFC1			
#define GLSI_KEY_F5                 0xFFC2			
#define GLSI_KEY_F6                 0xFFC3			
#define GLSI_KEY_F7                 0xFFC4			
#define GLSI_KEY_F8                 0xFFC5			
#define GLSI_KEY_F9                 0xFFC6
#define GLSI_KEY_F10                0xFFC7			
#define GLSI_KEY_F11                0xFFC8			
#define GLSI_KEY_F12                0xFFC9			

#define GLSI_KEY_SHIFT_L            0xFFE1    // Left shift 
#define GLSI_KEY_SHIFT_R            0xFFE2    // Right shift 
#define GLSI_KEY_CONTROL_L          0xFFE3    // Left control 
#define GLSI_KEY_CONTROL_R          0xFFE4    // Right control 
#define GLSI_KEY_CAPS_LOCK          0xFFE5    // Caps lock 
#define GLSI_KEY_SHIFT_LOCK         0xFFE6    // Shift lock 
#define GLSI_KEY_META_L             0xFFE7    // Left meta 
#define GLSI_KEY_META_R             0xFFE8    // Right meta 
#define GLSI_KEY_ALT_L              0xFFE9    // Left alt 
#define GLSI_KEY_ALT_R              0xFFEA    // Right alt 
#define GLSI_KEY_SUPER_L            0xFFEB    // Left super 
#define GLSI_KEY_SUPER_R            0xFFEC    // Right super 
#define GLSI_KEY_HYPER_L            0xFFED    // Left hyper 
#define GLSI_KEY_HYPER_R            0xFFEE    // Right hyper 

#define GLSI_KEY_TO_ASCII(key)      ((key) & 0x7f) 
#define GLSI_ASCII_TO_KEY(ascii)     (((ascii) & 0x7f) | 0xff80) 

typedef struct _glsiKeyBinding
{
  CHAR *pcKey;
  INT_32 iValue;
} glsiKeyBinding;

const UINT_32 gcuiglsiKeyBindings = 163;
extern glsiKeyBinding gcpglsiKeyBindings[gcuiglsiKeyBindings];

#endif // __STDINPUT_H__

